@@ -22,7 +22,11 @@ double probks(double lam) ;
 
 double nordis(double z) ;
 double ndens(double val, double mean, double sig) ;
+double lognordis(double z) ;
+double logndens(double val, double mean, double sig) ;
+
 double ntail(double z) ;
+void tailstats(double *x, double a, int isupper) ; 
 double zprob(double p) ;
 void setzptable() ;
 double z2x2(double *a) ;
@@ -41,6 +45,8 @@ double bessi1(double x) ;
 void bernload() ;
 double bernum(int x) ;
 
+void gpars(double *p, double *lam, double mean, double var) ;
+void mlegamma(double *a, int n, double *p, double *lam) ;
 void mleg(double a1, double a2, double *p, double *lam) ;
 
 double dilog(double x) ;
@@ -52,6 +58,7 @@ double gammprob(double x, double p, double lam)  ;
 double bprob(double p, double a, double b)  ;
 double lbeta(double a, double b)  ;
 double poissloglike(int kk, double mean) ;
+double poissexp(int kk, double mean) ;
 double dirmult(double *pp, int *aa, int len) ;
 double dawson(double t) ;
 
@@ -113,3 +120,21 @@ void gentail(double **ltail, double **rtail, double **hp, int a, int b)  ;
 void setthresh(int *thresh, double **tail, int a, int b, double stat, int mode)  ; 
 void bj2x(int *type, int a, int b, double *plpv, double *prpv, double *ppv)  ; 
 
+void mlebeta(double *a, int n, double *p1, double *p2);
+void estbpars(double *a, int n, double *p1, double *p2);
+void mleb(double *p1, double *p2, double u, double v) ;
+
+int  loadmptable(double ***mptable) ; 
+double wynn(double *v, int n, double *acc, int *nacc) ;
+double *vwynn(double **vv, int n, int dim, double **acc, int *nacc) ;
+
+double rad2deg(double rad) ; 
+double deg2rad(double deg) ; 
+
+double quartile(double *x, int n, double q)  ;
+int qinterp(double *a, double *b, int n, double val, double *ans)  ;
+double truncexpmean(double m, double thresh, int isupper) ;
+
+void jitter(double *xout, double *xin, int n)   ; 
+void mannwhit(double *a, int na, double *b, int nb, double *pu, double *pv, double *ppv) ;
+
