@@ -13,20 +13,29 @@ char *fwhite (char *ss) ;
 char *ftab (char *ss) ;
 int NPisnumber (char c) ;
 int isnumword (char *str)  ;
+void ffprint (FILE *fff, char *fmt, ...) ; 
+void enuf( char *fmt, ...) ;
 void fatalx( char *fmt, ...) ;
+int docommand( char *fmt, ...) ;
 long seednum() ;
 void printbl(int n) ;
 void printnl() ;
+void striplead(char *sss, char c) ;
 void striptrail(char *sss, char c) ;
 void catx(char *sout, char **spt, int n) ;
 void catxx(char *sout, char **spt, int n) ;
 void catxc(char *sout, char **spt, int n, char c) ;
 void makedfn(char *dirname, char *fname, char *outname, int maxstr) ;
 int substring (char **ap, char *inx, char *outx) ;
+int mapstrings(char **pstr, char **insub, char **outsub, int n)  ;
+int upstring (char *ss)  ; 
 int numcols (char *name) ;
 int numlines(char *name) ;
+int openit_trap (char *name, FILE ** fff, char *type); 
 void openit(char *name, FILE **fff, char *type)  ;
 int  ftest(char *aname) ;
+void fcheckr(char *name) ;
+void fcheckw(char *name) ; 
 int getxx(double **xx, int maxrow, int numcol, char *fname) ;
 int getss(char  **ss, char *fname) ;
 int  loadlist(char **list, char *listname)    ;  // with dup check
@@ -55,6 +64,7 @@ void freestring (char **ss) ;
 void copystrings(char **sa, char **sb, int n) ;
 void printstringsw(char **ss, int n, int slen, int width)  ;
 void printstrings(char **ss, int n)  ;
+void printstringsx(char **ss, int n)  ;
 int ridfile(char *fname) ; 
 char compbase(char x) ;
 void mkupper(char *sx) ;
@@ -76,8 +86,15 @@ char *findupper(char *s) ;
 char *fgetstrap(char *buff, int maxlen, FILE *fff, int *ret)  ;
 char readtonl(FILE *fff) ; 
 int  filehash(char *name) ;
-
-
+char *mytemp (char *qqq) ; 
+void printslurmenv ()  ; 
+int getfline(char *ss, char *fname, int maxstr) ;
+int copyfs(char *infile, FILE *fff)  ;
+int getxxq(double **xx, int maxrow, int numcol, char *fname) ; 
+int numcolsq (char *name) ;
+int getdata(char *buff, int nbytes, char *fname)  ;
+int putdata(char *buff, int nbytes, char *fname)  ;
+void writestrings(char *fname, char **ss, int n)  ;
 
 
 #define ZALLOC(item,n,type)      if ((item = (type *)calloc((n),sizeof(type))) == NULL) \
