@@ -16,8 +16,7 @@
 #include "egsubs.h"
 #include "exclude.h"
 
-#define WVERSION   "4510"
-
+#define WVERSION   "5000"
 /** 
  reformats files.             
  pedfile junk (6, 7 cols, ACGT added)
@@ -87,7 +86,9 @@
  familypopnames (use egroup for family in .fam output
  dupcheck contains iter number
 
- minvalpop :: every pop must have at least this number of valids (defualt not set) 
+ minvalpop :: every pop must have at least this number of valids (default not set) 
+ fillmissing: added 
+ better handling of seed
 */
 
 
@@ -99,8 +100,8 @@ int qtmode = NO;
 Indiv **indivmarkers, **indm2;
 SNP **snpmarkers;
 SNP **snpm2;
-int zerodistance = NO;          // YES => force gdis 0
-int downsample = NO;            // make pseudo homozygotes
+int zerodistance = NO;		// YES => force gdis 0
+int downsample = NO;		// make pseudo homozygotes
 int pordercheck = YES;
 int familypopnames = NO;
 
@@ -125,13 +126,16 @@ int flipreference = YES;
 int remapcheck = YES;
 
 char *poplistname = NULL;
+char *fillmissingpoplistname = NULL ; 
+char **fpops ;
+int nfpops = 0 ;
 
 double r2thresh = -1.0;
-double r2genlim = 0.01;         // Morgans 
+double r2genlim = 0.01;		// Morgans 
 double r2physlim = 5.0e6;
-double maxmissfrac = 1000.0;    // no thresh
-int maxmiss = -1;               // no thresh
-int minvalpop = -1;
+double maxmissfrac = 1000.0;	// no thresh
+int maxmiss = -1;		// no thresh
+int minvalpop = -1 ;
 int killr2 = NO;
 int mkdiploid = NO;
 
@@ -162,9 +166,9 @@ int hipos = 999999999;
 int minchrom = 1;
 int maxchrom = 97;
 
-int deletedup = YES;            // only one marker at a position
-char *newsnpname = NULL;        // new map  
-int newignore = YES;            // default ignore snps not in old list
+int deletedup = YES;		// only one marker at a position
+char *newsnpname = NULL;	// new map  
+int newignore = YES;		// default ignore snps not in old list
 int polarcheck = NO;
 int copyalleles = NO;
 
@@ -177,18 +181,18 @@ double nhwfilter = -1;
 void setomode (enum outputmodetype *outmode, char *omode);
 void readcommands (int argc, char **argv);
 void outfiles (char *snpname, char *indname, char *gname, SNP ** snpm,
-               Indiv ** indiv, int numsnps, int numind, int packem,
-               int ogmode);
+	       Indiv ** indiv, int numsnps, int numind, int packem,
+	       int ogmode);
 void remap (SNP ** s1, int nums1, SNP ** s2, int nums2);
 void remapind (SNP ** snpmarkers, int numsnps, Indiv ** indivmarkers,
-               Indiv ** indm2, int numindivs, int numind2);
+	       Indiv ** indm2, int numindivs, int numind2);
 void pickx (SNP * c1, SNP * c2, SNP ** px1, SNP ** px2);
 void dedupit (SNP ** snpmarkers, int numsnps);
 void flipsnps (char *fsname, SNP ** snpm, int numsnps, int phasedmode);
 void flipstrand (char *fsname, SNP ** snpm, int numsnps);
 int mkindh2d (Indiv ** indivmarkers, Indiv *** pindm2, int numindivs);
 void remaph2d (SNP ** snpmarkers, int numsnps, Indiv ** indivmarkers,
-               Indiv ** indm2, int numindivs, int numind2);
+	       Indiv ** indm2, int numindivs, int numind2);
 void flip1 (SNP * cupt, int phasedmode, int flipreference);
 
 void fixaa (SNP * cupt1, SNP * cupt2);
@@ -196,8 +200,8 @@ void fvalg (SNP * cupt, int val);
 char cxx (char *c1, char *c2);
 void downsamp (SNP * cupt);
 int setsamp (Indiv ** indivmarkers, int numindivs, char *usesamples);
-int testmisspop (SNP ** snpmarkers, int numsnps, Indiv ** indivmarkers,
-                 int numindivs, int minvalpops);
+int testmisspop(SNP **snpmarkers, int numsnps, Indiv **indivmarkers, int numindivs, int minvalpops)   ;
+int fillmiss(SNP **snpmarkers, Indiv **indivmarkers, int numsnps, int numindivs, char **fpops, int nfpops)  ;
 
 
 
@@ -234,20 +238,26 @@ main (int argc, char **argv)
   char c1, c2;
   int t1, t2;
 
-  malexhet = YES;               // convertf default is don't change the data
-  tersem = YES;                 // no snp counts
+  malexhet = YES;		// convertf default is don't change the data
+  tersem = YES;			// no snp counts
 
   readcommands (argc, argv);
 
-  if (fastdup)
+  
+  printf("## %s version: %s\n", argv[0], WVERSION) ;
+
+  if (seed == 0)  {
+   seed = seednum() ; 
+  }
+
+  SRAND(seed) ; 
+
+  if (fastdup) { 
     randommode = YES;
-  if ((randommode) && (seed == 0)) {
-    seed = seednum ();
-    printf ("seed: %d\n", seed);
   }
 
   if (randommode)
-    SRAND (seed);
+    printf("seed: %d\n", seed) ;
 
   if (chimpmode) {
     setchimpmode (YES);
@@ -310,13 +320,13 @@ main (int argc, char **argv)
 
   if (genotypelist != NULL) {
     getgenos_list (genotypelist, snpmarkers, indivmarkers,
-                   numsnps, numindivs, nignore);
+		   numsnps, numindivs, nignore);
   }
 
   else {
     setgenotypename (&genotypename, indivname);
     getgenos (genotypename, snpmarkers, indivmarkers,
-              numsnps, numindivs, nignore);
+	      numsnps, numindivs, nignore);
   }
 
   if (newsnpname != NULL) {
@@ -353,7 +363,7 @@ main (int argc, char **argv)
 
 
   if (deletedup)
-    dedupit (snpmarkers, numsnps);      // only one marker per position
+    dedupit (snpmarkers, numsnps);	// only one marker per position
 
   for (i = 0; i < numsnps; i++) {
     cupt = snpmarkers[i];
@@ -363,16 +373,14 @@ main (int argc, char **argv)
     c1 = cupt->alleles[0];
     c2 = cupt->alleles[1];
     t1 = pedval (&c1) % 5;
-    t2 = pedval (&c2) % 5;      // 0 and 5 are no good
+    t2 = pedval (&c2) % 5;	// 0 and 5 are no good
     if ((t1 == 0) && (t2 > 0))
       flip1 (cupt, phasedmode, YES);
   }
 
-  flipstrand (flipstrandname, snpmarkers, numsnps);
-  flipsnps (flipsnpname, snpmarkers, numsnps, phasedmode);
 
   if (deletedup)
-    dedupit (snpmarkers, numsnps);      // only one marker per position
+    dedupit (snpmarkers, numsnps);	// only one marker per position
 
   for (i = 0; i < numsnps; i++) {
     cupt = snpmarkers[i];
@@ -382,7 +390,7 @@ main (int argc, char **argv)
     c1 = cupt->alleles[0];
     c2 = cupt->alleles[1];
     t1 = pedval (&c1) % 5;
-    t2 = pedval (&c2) % 5;      // 0 and 5 are no good
+    t2 = pedval (&c2) % 5;	// 0 and 5 are no good
     if ((t1 == 0) && (t2 > 0))
       flip1 (cupt, phasedmode, YES);
   }
@@ -395,17 +403,17 @@ main (int argc, char **argv)
       cupt = snpmarkers[i];
       g = getgtypes (cupt, polarindex);
       if (g == 0) {
-        printf ("polarizing %s", cupt->ID);
-        printf (" %3d %12.0f", cupt->chrom, cupt->physpos);
-        printnl ();
-        fflush (stdout);
-        flip1 (cupt, NO, YES);
-        g = getgtypes (cupt, polarindex);
-        if (g != 2)
-          fatalx ("badbug\n");
+	printf ("polarizing %s", cupt->ID);
+	printf (" %3d %12.0f", cupt->chrom, cupt->physpos);
+	printnl ();
+	fflush (stdout);
+	flip1 (cupt, NO, YES);
+	g = getgtypes (cupt, polarindex);
+	if (g != 2)
+	  fatalx ("badbug\n");
       }
       if (g != 2)
-        cupt->ignore = YES;
+	cupt->ignore = YES;
     }
   }
   for (i = 0; i < numsnps; i++) {
@@ -416,7 +424,7 @@ main (int argc, char **argv)
 
   if (outputall) {
     outfiles (snpoutfilename, indoutfilename, genooutfilename,
-              snpmarkers, indivmarkers, numsnps, numindivs, packout, ogmode);
+	      snpmarkers, indivmarkers, numsnps, numindivs, packout, ogmode);
 
     printf ("##end of convertf run (outputall mode)\n");
     return 0;
@@ -425,9 +433,16 @@ main (int argc, char **argv)
   if (usesamples != NULL) {
     poplistname = NULL;
     setsamp (indivmarkers, numindivs, usesamples);
-
-
   }
+
+  if (fillmissingpoplistname != NULL) {  
+   printf("seed: %d\n", seed) ;
+   ZALLOC(fpops, numindivs, char *) ; 
+   nfpops = loadlist(fpops, fillmissingpoplistname) ;
+   t = fillmiss(snpmarkers, indivmarkers, numsnps, numindivs, fpops, nfpops) ;
+   printf("%10d missing genotypes filled\n", t) ;
+  }
+
   if (poplistname != NULL) {
     ZALLOC (eglist, numindivs, char *);
     numeg = loadlist (eglist, poplistname);
@@ -435,12 +450,13 @@ main (int argc, char **argv)
     for (i = 0; i < numindivs; ++i) {
       indx = indivmarkers[i];
       if (indx->affstatus == NO)
-        indx->ignore = YES;
+	indx->ignore = YES;
     }
   }
   else
     setstatus (indivmarkers, numindivs, "Case");
 
+
   numsnps = rmsnps (snpmarkers, numsnps, deletesnpoutname);
   numindivs = rmindivs (snpmarkers, numsnps, indivmarkers, numindivs);
 //  printf("got here! 2\n") ; fflush(stdout) ;
@@ -448,7 +464,7 @@ main (int argc, char **argv)
   if (killr2) {
     nkill =
       killhir2 (snpmarkers, numsnps, numindivs, r2physlim, r2genlim,
-                r2thresh);
+		r2thresh);
     if (nkill > 0)
       printf ("killhir2.  number of snps killed: %d\n", nkill);
   }
@@ -482,9 +498,8 @@ main (int argc, char **argv)
   if (numvalidind == 0)
     fatalx ("no valid samples!\n");
 
-  t = testmisspop (snpmarkers, numsnps, indivmarkers, numindivs, minvalpop);
-  if (minvalpop > 0)
-    printf ("minvalpop: deleted %d   retained %d\n", t, numsnps - t);
+  t = testmisspop(snpmarkers, numsnps, indivmarkers, numindivs, minvalpop) ; 
+  if (minvalpop>0) printf("minvalpop: deleted %d   retained %d\n", t, numsnps-t) ;
 
 
   for (k = 0; k < numsnps; ++k) {
@@ -495,7 +510,6 @@ main (int argc, char **argv)
     if (maxmiss < t) {
       cupt->ignore = YES;
     }
-
 /**
    if (numvalidind ==  t) { 
     printf("no data for snp: %s\n", cupt -> ID) ;
@@ -525,7 +539,7 @@ main (int argc, char **argv)
   }
 
   outfiles (snpoutfilename, indoutfilename, genooutfilename,
-            snpmarkers, indivmarkers, numsnps, numindivs, packout, ogmode);
+	    snpmarkers, indivmarkers, numsnps, numindivs, packout, ogmode);
 
   printf ("##end of convertf run\n");
   return 0;
@@ -584,11 +598,11 @@ output:        eurout
   getstring (ph, "flipsnpname:", &flipsnpname);
   getstring (ph, "flipstrandname:", &flipstrandname);
   getstring (ph, "indoutfilename:", &indoutfilename);
-  getstring (ph, "indivoutname:", &indoutfilename);     /* changed 11/02/06 */
+  getstring (ph, "indivoutname:", &indoutfilename);	/* changed 11/02/06 */
   getstring (ph, "snpoutfilename:", &snpoutfilename);
-  getstring (ph, "snpoutname:", &snpoutfilename);       /* changed 11/02/06 */
+  getstring (ph, "snpoutname:", &snpoutfilename);	/* changed 11/02/06 */
   getstring (ph, "genooutfilename:", &genooutfilename);
-  getstring (ph, "genotypeoutname:", &genooutfilename); /* changed 11/02/06 */
+  getstring (ph, "genotypeoutname:", &genooutfilename);	/* changed 11/02/06 */
   getstring (ph, "outputformat:", &omode);
   getstring (ph, "outputmode:", &omode);
   getstring (ph, "polarize:", &polarid);
@@ -611,10 +625,10 @@ output:        eurout
 
   getint (ph, "outputgroup:", &ogmode);
   getint (ph, "malexhet:", &malexhet);
-  getint (ph, "nomalexhet:", &malexhet);        /* changed 11/02/06 */
+  getint (ph, "nomalexhet:", &malexhet);	/* changed 11/02/06 */
   getint (ph, "tersemode:", &tersem);
   getint (ph, "familynames:", &familynames);
-  getint (ph, "packout:", &packout);    /* now obsolete 11/02/06 */
+  getint (ph, "packout:", &packout);	/* now obsolete 11/02/06 */
   getint (ph, "decimate:", &decim);
   getint (ph, "dmindis:", &dmindis);
   getint (ph, "dmaxdis:", &dmaxdis);
@@ -647,6 +661,7 @@ output:        eurout
   getint (ph, "minvalpop:", &minvalpop);
 
   getstring (ph, "poplistname:", &poplistname);
+  getstring (ph, "fillmissingpoplistname:", &fillmissingpoplistname);
   getstring (ph, "newsnpname:", &newsnpname);
   getint (ph, "newignore:", &newignore);
   getstring (ph, "newindivname:", &newindivname);
@@ -671,7 +686,7 @@ remap (SNP ** s1, int nums1, SNP ** s2, int nums2)
     if (k < 0) {
       printf ("%20s not found\n", cupt2->ID);
       if (newignore) {
-        cupt2->ignore = YES;
+	cupt2->ignore = YES;
       }
       continue;
     }
@@ -706,7 +721,7 @@ cxx (char *c1, char *c2)
   if (c1[0] == c2[1])
     return c2[0];
 
-  return 'X';
+  return 'X' ; 
 
 }
 
@@ -744,7 +759,7 @@ fixaa (SNP * cupt1, SNP * cupt2)
       ok = NO;
     if (ok == NO) {
       printf ("forcing all genos invalid for %s %c %c     %c %c\n", cupt1->ID,
-              c1[0], c1[1], c2[0], c2[1]);
+	      c1[0], c1[1], c2[0], c2[1]);
       fvalg (cupt1, 999);
       cupt1->ignore = YES;
       return;
@@ -765,13 +780,13 @@ fixaa (SNP * cupt1, SNP * cupt2)
   }
   if ((c1[1] == c2[1]) && (c1[0] != c2[0])) {
     t = 0;
-    fvalg (cupt1, 0);           // only valid genotype ;
+    fvalg (cupt1, 0);		// only valid genotype ;
     return;
   }
-  fvalg (cupt1, t);             // force all snps invalid
+  fvalg (cupt1, t);		// force all snps invalid
   if (t == 999) {
     printf ("forcing all genos invalid for %s %c %c     %c %c\n", cupt1->ID,
-            c1[0], c1[1], c2[0], c2[1]);
+	    c1[0], c1[1], c2[0], c2[1]);
   }
 }
 
@@ -811,7 +826,7 @@ fvalg (SNP * cupt, int val)
 
 void
 remapind (SNP ** snpmarkers, int numsnps, Indiv ** indivmarkers,
-          Indiv ** indm2, int numindivs, int numind2)
+	  Indiv ** indm2, int numindivs, int numind2)
 {
 
   int *g1, *g2, *w1;
@@ -819,12 +834,12 @@ remapind (SNP ** snpmarkers, int numsnps, Indiv ** indivmarkers,
   Indiv *indx;
   SNP *cupt;
 
-  if (numindivs != numind2)
+  if ((numindivs != numind2) && (remapcheck == YES)) 
     fatalx ("different remapind sizes %d %d\n", numindivs, numind2);
   ZALLOC (tind, numind2, int);
-  ZALLOC (g2, numind2, int);
-  ZALLOC (g1, numind2, int);
-  ZALLOC (w1, numind2, int);
+  ZALLOC (g2, numindivs, int);
+  ZALLOC (g1, numindivs, int);
+  ZALLOC (w1, numindivs, int);
 
   for (k = 0; k < numind2; ++k) {
     indx = indm2[k];
@@ -836,16 +851,15 @@ remapind (SNP ** snpmarkers, int numsnps, Indiv ** indivmarkers,
   for (i = 0; i < numsnps; i++) {
     cupt = snpmarkers[i];
 
-    for (j = 0; j < numind2; ++j) {
+    for (j = 0; j < numindivs; ++j) {
       g1[j] = getgtypes (cupt, j);
     }
-    copyiarr (g1, w1, numind2);
+    copyiarr (g1, w1, numindivs);
 
     for (k = 0; k < numind2; ++k) {
       g2[k] = g1[tind[k]];
     }
 
-    ivclear (g1, -1, numind2);
     copyiarr (g2, g1, numind2);
 
     for (k = 0; k < numind2; ++k) {
@@ -888,7 +902,7 @@ dedupit (SNP ** snpmarkers, int numsnps)
       cupt1 = cupt2;
       continue;
     }
-    pickx (cupt1, cupt2, &x1, &x2);     // x2 bad
+    pickx (cupt1, cupt2, &x1, &x2);	// x2 bad
     x2->ignore = YES;
     cupt1 = x1;
   }
@@ -1013,13 +1027,13 @@ flip1 (SNP * cupt, int phasedmode, int flipreference)
 
 void
 setsamplist (Indiv ** indivmarkers, int numindivs, char **samplist,
-             int nsplit)
+	     int nsplit)
 {
   int i, j, k;
   Indiv *indx;
   int t = 0;
 
-  setstatusv (indivmarkers, numindivs, NULL, NO);       // set affstatus to NO
+  setstatusv (indivmarkers, numindivs, NULL, NO);	// set affstatus to NO
 
   for (j = 0; j < nsplit; ++j) {
     k = indindex (indivmarkers, numindivs, samplist[j]);
@@ -1058,86 +1072,163 @@ setsamp (Indiv ** indivmarkers, int numindivs, char *usesamples)
   return nsplit;
 
 }
-
-int
-mkpops (char **pops, Indiv ** indm, int numind)
+int mkpops(char **pops, Indiv **indm, int numind) 
 // make a list of pops from indm
 {
-  int n = 0;
-  Indiv *indx;
-  int k, t;
-  char *sx;
-
-  for (k = 0; k < numind; ++k) {
-    indx = indivmarkers[k];
-    if (indx->ignore)
-      continue;
-    sx = indx->egroup;
-    t = indxstring (pops, n, sx);
-    if (t >= 0)
-      continue;
-    pops[n] = strdup (sx);
-    ++n;
+  int n = 0 ;
+  Indiv *indx ; 
+  int k, t ; 
+  char *sx ; 
+ 
+  for (k=0; k<numind; ++k) { 
+   indx = indivmarkers[k] ;
+   if (indx -> ignore) continue ;
+   sx = indx -> egroup ; 
+   t = indxstring(pops, n, sx) ; 
+   if (t>=0) continue ; 
+   pops[n] = strdup(sx) ;
+   ++n ; 
   }
 // make a list of pops from indm
 
-  return n;
+  return n ; 
 
 
 }
-
 int
-testmisspop (SNP ** snpmarkers, int numsnps, Indiv ** indivmarkers,
-             int numindivs, int minvalpops)
+testmisspop(SNP **snpmarkers, int numsnps, Indiv **indivmarkers, int numindivs, int minvalpops)  
 {
+    
+  char **pops ; 
+  int  npops, ndelete = 0 ;
+  int **valcnt, i, k, x, g ; 
+  Indiv *indx ; 
+  SNP *cupt ;
 
-  char **pops;
-  int npops, ndelete = 0;
-  int **valcnt, i, k, x, g;
-  Indiv *indx;
-  SNP *cupt;
-
-  if (minvalpops < 1)
-    return 0;
-  ZALLOC (pops, numindivs, char *);
-  npops = mkpops (pops, indivmarkers, numindivs);
-  if (npops == 0)
-    fatalx ("no valid data\n");
+  if (minvalpops<1) return 0 ; 
+  ZALLOC(pops, numindivs, char *) ; 
+  npops = mkpops(pops, indivmarkers, numindivs) ;
+  if (npops == 0) fatalx("no valid data\n") ;
   if (verbose) {
-    printf ("zzpops:\n");
-    printstrings (pops, npops);
+   printf("zzpops:\n") ; printstrings(pops, npops) ;
   }
 
-  valcnt = initarray_2Dint (numsnps, npops, 0);
-  for (i = 0; i < numindivs; ++i) {
-    indx = indivmarkers[i];
-    if (indx->ignore)
-      continue;
-    x = indxstring (pops, npops, indx->egroup);
-    if (x < 0)
-      continue;
-    for (k = 0; k < numsnps; ++k) {
-      cupt = snpmarkers[k];
-      if (cupt->ignore)
-        continue;
-      g = getgtypes (cupt, i);
-      if (g < 0)
-        continue;
-      ++valcnt[k][x];
-    }
+  valcnt = initarray_2Dint(numsnps, npops, 0) ; 
+  for (i=0; i<numindivs; ++i) { 
+   indx = indivmarkers[i] ; 
+   if (indx -> ignore) continue ; 
+   x = indxstring(pops, npops, indx -> egroup) ; 
+   if (x<0) continue ; 
+   for (k=0; k<numsnps; ++k) { 
+    cupt = snpmarkers[k] ; 
+    if (cupt -> ignore) continue ; 
+    g = getgtypes(cupt, i) ; 
+    if (g<0) continue ; 
+    ++valcnt[k][x] ; 
+   }
   }
-  for (k = 0; k < numsnps; ++k) {
-    if (verbose && (k < 10))
-      printimat (valcnt[k], 1, npops);
-    ivmaxmin (valcnt[k], npops, NULL, &x);
-    if (x < minvalpops) {
-      snpmarkers[k]->ignore = YES;
-      ++ndelete;
-    }
+  for (k=0; k<numsnps; ++k) { 
+   if (verbose && (k<10))  printimat(valcnt[k], 1, npops) ;
+   ivmaxmin(valcnt[k], npops, NULL, &x) ; 
+   if (x<minvalpops) { 
+     snpmarkers[k] -> ignore = YES ; 
+     ++ndelete ; 
+   }
   }
 
-  freeup (pops, numindivs);
-  free2Dint (&valcnt, numsnps);
-  return ndelete;
+ freeup(pops, numindivs) ; 
+ free2Dint(&valcnt, numsnps) ;
+ return ndelete ; 
+
+}
+
+int count_column(SNP *cupt, int *xtypes, int numindivs, int **ncount, int *nmiss, int npops) 
+// ncount npops x 3     nmiss ncount long   must be pre-allocated
+{
+    int i, k, g  ;
+
+    if (cupt -> ignore)  return -1 ;
+
+    iclear2D(&ncount, npops, 3, 0) ;
+    ivzero(nmiss, npops) ;
+
+    for (i=0; i<numindivs; ++i) { 
+     k = xtypes[i]  ; 
+     if (k<0) continue ; 
+     if (k>= npops) fatalx("(count_column) overflow from xtypes\n") ;  
+     g = getgtypes(cupt, i) ;
+     if (g<0) {    
+      ++nmiss[k] ;
+      continue ;
+     }
+     ++ncount[k][g] ; 
+    }
+
+   return intsum(nmiss, npops) ; 
+
+}
+
+int fillmiss(SNP **snpmarkers, Indiv **indivmarkers, int numsnps, int numindivs, char **fpops, int nfpops) 
+
+{
+
+   int *xtypes, i, j, k, g, tmiss, g1, g2, l ; 
+   int top, bot ;
+   Indiv *indx ; 
+   SNP *cupt ;
+   int **ncount, *nmiss, *nc ;
+   int nfill = 0 ;
+   double y ; 
+
+   if (nfpops == 0) return 0 ; 
+   if (fpops == NULL) return 0 ; 
+
+   ZALLOC(xtypes, numindivs, int) ; 
+   ZALLOC(nmiss, nfpops, int) ; 
+
+   for (i=0; i<numindivs; ++i) {  
+    indx = indivmarkers[i] ; 
+    k = indxindex(fpops, nfpops, indx -> egroup) ; 
+    xtypes[i] = k ; 
+   }
+
+   ncount = initarray_2Dint(nfpops, 3, 0) ;  // 0, 1 2  count
+
+   for (j=0; j<numsnps; ++j) { 
+    cupt = snpmarkers[j] ; 
+    if (cupt -> ignore) continue ;
+
+    tmiss  = count_column(cupt, xtypes, numindivs, ncount, nmiss, nfpops) ;
+
+    for (k=0; k<nfpops; ++k) {  
+     if (cupt -> ignore) break ; 
+     if (tmiss==0) break ; 
+     nc = ncount[k] ; 
+     bot = 2*intsum(nc, 3) ;         
+     if (bot==0) {  
+      cupt -> ignore = YES;  
+      continue ;
+     }
+    if (nmiss[k] <= 0) continue ;  // no fill in needed
+     top = 2*nc[2] + nc[1] ; 
+     y = (double) top / (double) bot ; 
+// now fill in 
+     for (i=0; i<numindivs; ++i) { 
+      if (xtypes[i] != k) continue ; 
+      g = getgtypes(cupt, i) ;
+      if (g>=0) continue ; 
+      g1 = prob1(y) ; 
+      g2 = prob1(y) ; 
+      putgtypes(cupt, i, g1 + g2) ;
+      ++nfill ;
+     }
+    } 
+   }
+
+   free2Dint(&ncount, nfpops) ; 
+   free(xtypes) ;
+   free(nmiss) ;
+
+   return nfill ;
 
 }
