@@ -8,6 +8,7 @@
 
 #include "nicklib.h"
 #include "getpars.h"
+#include "strsubs.h" 
 
 /** 
  a very simple keyword parameter cracker 
@@ -72,8 +73,8 @@ openpars (char *fname)
         fatalx ("duplicate parameter: %s\n", ww);
       ppars[npars] = strdup (ww);
 
-      striptrail (rest, ' ');   /* no trailing blanks */
       stripcomment (rest);
+      striptrail (rest, ' ');   /* no trailing blanks */
       pdata[npars] = strdup (rest);
       ++npars;
 
@@ -95,11 +96,11 @@ openpars (char *fname)
 
 
   for (i = 0; i < npars; i++) {
+
     pp->ppars[i] = strdup (ppars[i]);
     pp->pdata[i] = strdup (pdata[i]);
-
-
-    /*  printf("zz: %d %s %s\n",i,ppars[i],pp->ppars[i]) ; */
+    striplead(pp -> pdata[i], ' ') ;
+//  printf("zz: %d %s %s\n",i,ppars[i],pp->ppars[i]) ; 
 
   }
 
@@ -152,6 +153,22 @@ stripcomment (char *str)
 
 #define MAXFIELD 1000
 
+int
+getlongstring (phandle * pp, char *parname, char **strng)
+// whole of line
+{
+
+  char *field[MAXFIELD];
+  int n, kode;
+
+  kode = findpname (pp, parname);
+  if (kode < 0)
+    return kode;
+  *strng = strdup (pp->pdata[kode]);
+  return 1;
+}
+
+
 int
 getstring (phandle * pp, char *parname, char **strng)
 {
@@ -431,25 +448,3 @@ dostrsub (phandle * pp)
 
 }
 
-int
-upstring (char *ss)
-
-/* 
- YES if at least one upper case character 
- and no lower case  
-*/
-{
-  int nupper = 0;
-  int i;
-  for (i = 0; i < strlen (ss); i++)
-  {
-    if (islower (ss[i]))
-      return NO;
-    if (isupper (ss[i]))
-      ++nupper;
-  }
-  if (nupper > 0)
-    return YES;
-  return NO;
-
-}
