@@ -7,7 +7,7 @@
 #include "vsubs.h"
 
 /** 
- a simple sort routine
+ a simple set of sort routines
 */
 
 static double *ttt;
@@ -219,6 +219,7 @@ invperm (int *a, int *b, int n)
 void
 ipsortit (int **a, int *ind, int len, int rlen)
 {
+  if (len==0) return ;
   ipsortitp (a, ind, len, rlen, NULL);
 
 }
@@ -256,6 +257,7 @@ ipsortitp (int **a, int *ind, int len, int rlen, int *order)
   for (i = 0; i < len; i++) {
     k = inda[i];
     a[i] = pttt[k];
+// note that this just reorders pointers.  
   }
   if (ind != NULL)
     copyiarr (inda, ind, len);
