@@ -24,7 +24,7 @@ static int numbox = QSIZE * ZLIM;
 
 
 
-static double zzprob (double zval);
+static double zzprob (double pval);
 static double znewt (double z, double ptail);
 
 static double ltlg1 (double a, double x);
@@ -46,13 +46,29 @@ static double **bcotable = NULL;
 static int bcosize = -1;	// max val of binomial coefficient  
 
 
+double
+lognordis (double zval)
+/* normal density */
+{
+  double pi, t;
+
+  pi = PI ;
+
+  t = -0.5 * zval * zval;
+  t -= 0.5*log (2.0 * pi);
+
+  return t;
+
+}
+
+
 double
 nordis (double zval)
 /* normal density */
 {
   double pi, t;
 
-  pi = 2.0 * acos (0.0);
+  pi = PI ;
 
   t = exp (-0.5 * zval * zval);
   t /= sqrt (2.0 * pi);
@@ -61,6 +77,12 @@ nordis (double zval)
 
 }
 
+double
+logndens (double val, double mean, double sig)
+{
+  return lognordis ((val - mean) / sig) - log(sig) ;
+}
+
 double
 ndens (double val, double mean, double sig)
 {
@@ -87,7 +109,7 @@ ntail (double zval)
       return q;
     }
 
-  pi = 2.0 * acos (0.0);
+  pi = PI; 
 
   t = exp (-0.5 * zval * zval);
   t /= (sqrt (2.0 * pi) * zval);
@@ -95,6 +117,28 @@ ntail (double zval)
   return t;
 
 }
+void tailstats(double *x, double a, int isupper) 
+// P(z>a) and conditional mean and variance if isupper = YES
+{
+  double p0, y1, y2, m, z1, z2 ;
+
+  if (isupper == NO) { 
+   tailstats(x, -a, YES) ; 
+   x[1] = -x[1] ; 
+   return ; 
+  }
+  p0 = x[0] = ntail(a) ; 
+  y1 = exp(-a*a/2.0) / sqrt(2.0*PI) ; 
+//  printf("zz %12.6f %12.6f\n", p0, y1) ;  
+  y2 = a*y1 + p0 ; 
+  z1 = m = x[1] = y1/p0 ; 
+  z2 = x[2] = y2/p0 - m*m ; 
+  if (!isfinite(z1) || (!isfinite(z2))) {
+   x[1] = a ; 
+   x[2] = 0 ; 
+  }
+  return ; 
+}
 
 double
 zzprob (double pval)
@@ -365,8 +409,7 @@ z2x2 (double *a)
 	      fatalx ("(conchi) zero row or column sum\n");
 	    }
 	  y = a[k] - ee;
-	  if (k == 0)
-	    dev00 = y;
+	  if (k == 0) dev00 = y;
 	  chsq += (y * y) / ee;
 	}
     }
@@ -593,6 +636,33 @@ firstgt (double val, double *tab, int n)
     return n ;
 }
 
+void gpars(double *p, double *lam, double mean, double var)
+{
+          double l ;
+
+          if (mean<0.0) fatalx("(gpars) bad mean\n") ;
+          if (var<0.0) fatalx("(gpars) bad var\n") ;
+
+          l = mean/var ;
+          *p = l*mean ;
+          *lam = l ;
+}
+
+
+
+void mlegamma(double *a, int n, double *p, double *lam) 
+{
+
+  double y1, y2 ;  
+
+  y1 = asum(a, n)/ (double) n ; 
+  y2 = logsum(a, n)/ (double) n ; 
+
+  mleg(y1, y2, p, lam) ;
+//  printf("zzmlegamm %d %9.3f %9.3f %9.3f %9.3f\n", n, y1, y2, *p, *lam) ;
+
+}
+
 void
 mleg (double a1, double a2, double *p, double *lam)
 {
@@ -972,6 +1042,7 @@ hwstat (double *x)
 
 double
 bprob (double p, double a, double b)
+// beta log density 
 {
   double q, yl;
   q = 1.0 - p;
@@ -984,12 +1055,18 @@ bprob (double p, double a, double b)
   return yl;
 }
 
+double qlog(double x) 
+{
+  return log(x) ;
+}
+
 double
 gammprob (double x, double p, double lam)
-/* gamma density */
+/* gamma log density */
 {
   double xx, yl;
   xx = MAX (x, 1.0e-8);
+//  printf("zzgp %9.3f %9.3f %9.3f\n", p, xx) ;
   yl = (p - 1) * log (xx) - lam * xx;
   yl += p * log (lam);
   yl -= xlgamma (p);
@@ -997,6 +1074,7 @@ gammprob (double x, double p, double lam)
 }
 
 
+
 double
 lbeta (double a, double b)
 {
@@ -1975,13 +2053,6 @@ bpars (double *a, double *b, double mean, double var)
   xb = x * (1 - m);
 
 
-/**
-  ym = xa/(xa+xb) ;  
-  yv = xa*xb/((xa+xb)*(xa+xb)*(xa+xb+1)) ;
-  printf("%9.3f %9.3f\n", mean, ym) ;  
-  printf("%9.3f %9.3f\n", var, yv) ;
-*/
-
   *a = xa;
   *b = xb;
 
@@ -1998,6 +2069,7 @@ bmoments (double a, double b, double *mean, double *var)
   *mean = a / x;
   *var = (a * b) / (x * x * (x + 1));
 
+
 }
 
 double
@@ -2150,13 +2222,8 @@ weightjackx (double *est, double *sig, double mean, double *jmean,
 
   vclear (hh, yn, g);
   vvd (hh, hh, jwt, g);
-/**
-  for (k=0; k<g; ++k) {
-   if (jwt[k] > 0.0) hh[k] /= jwt[k] ;  
-   else hh[k] *= 1.0e20 ;
-  }
-*/
-// jwt should be positive
+
+// jwt should be positive (forced by weightjack()) 
 
   vst (xtau, hh, mean, g);
   vsp (w1, hh, -1.0, g);
@@ -2533,9 +2600,6 @@ void bj2(double *aa, double *bb, int a, int b,  double *plpv, double *prpv, doub
  free(cat) ; 
  free(ind) ; 
  free(type) ; 
- 
-
-
 
 
 }
@@ -2747,3 +2811,529 @@ double genhpt(int a, int b, int *lt, int *rt)
   free2Dlongdouble(&hp, t) ;
   return 1.0-y ;  
 }
+
+void mlegamm(double *a, double n, double *p, double *lam)
+{
+
+  double a1, a2 ;
+  int k, num=0 ;
+
+  a1 = a2 = 0 ;
+  for (k=0; k<n; ++k) {
+   if (a[k] <= 0.0) continue ;
+   a1 += a[k] ;
+   a2 += log(a[k]) ;
+   ++num ;
+  }
+  a1 /= (double) num ;
+  a2 /= (double) num ;
+  mleg(a1, a2, p, lam) ;
+
+}
+
+void mlebeta(double *a, int n, double *p1, double *p2)
+
+{
+
+
+  double a1, a2 ;
+  int k, num=0 ;
+
+  a1 = a2 = 0 ;
+  estbpars(a, n,  p1, p2) ; //  initial moments estimator
+  for (k=0; k<n; ++k) {
+   if (a[k] <= 0.0) continue ;
+   if (a[k] >= 1.0) continue ;
+   a1 += log(a[k]) ;
+   a2 += log(1-a[k]) ;
+   ++num ;
+  }
+  a1 /= (double) num ;
+  a2 /= (double) num ;
+  mleb(p1, p2, a1, a2) ;
+
+}
+
+
+
+void estbpars(double *a, int n, double *p1, double *p2)
+
+{
+
+// moments estimator for beta
+
+  double *w1, ymean, yvar ;
+
+  ZALLOC(w1, n, double) ;
+
+  ymean = asum(a, n) / (double) n ;
+  vsp(w1, a, -ymean, n) ;
+  yvar = asum2(w1, n) / (double) n ;
+
+  bpars(p1, p2, ymean, yvar) ;
+
+  free(w1) ;
+
+
+}
+
+
+static void mleb1(double *p1, double *p2, double u, double mul)       
+{
+  double b1, b2, TT[1], R[1], H[1], X[1]  ;  
+  int iter, ret ;
+  double  fval ; 
+
+  b1 = *p1 ; 
+  b2 = *p2 ; 
+
+  for (iter = 1; iter <= 30; ++iter) {  
+   fval = u - (psi(b1) - psi(b1+b2)) ; 
+   if (fval <= 0) break ; 
+   b1 *= mul ;
+//  printf("zz1 %15.9f %12.6f\n", b1, fval) ;
+  }
+
+  for (iter = 1; iter <= 30; ++iter) {  
+   fval = u - (psi(b1) - psi(b1+b2)) ; 
+   if (fval >= 0) break ; 
+   b1 /= mul ;
+//   printf("zz2 %15.9f %12.6f\n", b1, fval) ;
+  }
+
+  *p1 = b1 ;
+
+}
+void mleb(double *p1, double *p2, double u, double v) 
+
+// u = mean(log x_i)  v = mean(log (1-x_i)) 
+{
+
+  double b1, b2, TT[4], R[2], H[2], X[2], bb1, bb2 ;  
+  int iter, ret ;
+  double yerr, oldyerr = 1.0e20 ; 
+
+  b1 = *p1 ; 
+  b2 = *p2 ; 
+
+   mleb1(&b1, &b2, u, 2) ; 
+   mleb1(&b2, &b1, v, 2) ; 
+
+   mleb1(&b1, &b2, u, 1.5) ; 
+   mleb1(&b2, &b1, v, 1.5) ; 
+
+  for (iter=1; iter <= 20; ++iter)  { 
+   if (oldyerr < yerr) { 
+    mleb1(&b1, &b2, u, 2) ; 
+    mleb1(&b2, &b1, v, 2) ; 
+    continue ;  
+   } 
+   R[0] = psi(b1)-psi(b1+b2) -u ;
+   R[1] = psi(b2)-psi(b1+b2) -v ;
+
+   yerr = fabs(R[0]) + fabs(R[1]) ;
+
+   
+   TT[0] = tau(b1) - tau(b1+b2) ;
+   TT[3] = tau(b2) - tau(b1+b2) ;
+   TT[1] = TT[2] = -tau(b1+b2) ;  
+
+   vst(X, R, -1, 2) ;
+
+/**
+   printf("iter: %3d %15.9f %15.9f\n", iter, b1, b2) ;
+   printmatl(R, 1, 2) ; 
+   printmatl(TT, 2, 2) ;  
+*/
+   ret = solvit(TT, X, 2, H) ;  
+   if (ret<0) fatalx("(mleb) TT not pos def\n") ;
+// printmatl(H, 1, 2) ; 
+   bb1 = b1 + H[0] ;
+   bb2 = b2 + H[1] ;
+   if (bb1<0) bb1 = b1/2 ; 
+   if (bb2<0) bb2 = b2/2 ; 
+   b1 = bb1 ; 
+   b2 = bb2 ;
+   if (yerr < 1.0e-12) break ;
+   oldyerr = yerr ;
+
+  }
+  *p1 = b1 ; 
+  *p2 = b2 ; 
+
+}
+
+int loadmptable(double ***mptab) 
+// return number of rows in table 
+{
+
+ int n, nr, nc, i, j, t ; 
+ static double **tab ; 
+
+ n = MPTABSIZE ; 
+ 
+ nr = n/6 ; nc = 6 ; 
+
+ tab = initarray_2Ddouble(nc, nr, 0) ; 
+ t = 0 ; 
+ for (i=0; i<nr ; ++i) { 
+  for (j=0; j<nc ; ++j) { 
+   tab[j][i] = MPTABLE[t] ;
+   ++t ; 
+ }} 
+ 
+ *mptab = tab ; 
+ return nr ; 
+/** 
+ 0  g 
+ 1 lq
+ 2 med
+ 3 up
+ 4 r 
+ 5 r/tau = sqrt(g) 
+*/
+
+}
+
+int qinterp(double *a, double *b, int n, double val, double *ans) 
+/**
+ a and b are matched values 
+ sort a and corresponding b  Set val as linear combo of 2 a values 
+ return linear combo of corresponding b values 
+ called by quartile 
+ return approx index in sorted list.  -1 if lo n+1000 if hi
+*/
+
+{ 
+ 
+ double *xa, *xb ; 
+ int *ind, k, x  ; 
+ double y1,y2, yy ;
+
+ ZALLOC(xa, n+2, double) ; 
+ ZALLOC(xb, n+2, double) ; 
+ ZALLOC(ind, n+2, int) ; 
+
+ copyarr(a, xa+1, n) ; 
+ copyarr(b, xb+1, n) ; 
+
+ 
+ sortit(xa+1, ind, n) ; 
+
+ for (k=0; k<n ;  ++k) { 
+  x = ind[k] ; 
+  xb[k+1] = b[x] ; 
+ } 
+
+ xb[0] = xb[1] ; 
+ xb[n+1] = xb[n] ; 
+ xa[0] = xa[1]  -  1.0e20 ; 
+ xa[n+1] = xa[n] + 1.0e20 ; 
+
+ x = firstgtx(val, xa, n+2) ;  
+
+ if (x<=1) { 
+  *ans = xb[0] ;
+  free(xa) ; 
+  free(xb) ; 
+  free(ind) ; 
+  return -1 ; 
+ }
+ if (x==(n+1)) { 
+  *ans = xb[n] ;
+  free(xa) ; 
+  free(xb) ; 
+  free(ind) ; 
+  return n + 1000 ;
+ }
+ y1 = xa[x] - val ; 
+ y2 = val - xa[x-1] ; 
+ yy = y1 + y2 ; 
+
+ y1 /= yy ; 
+ y2 /= yy ; 
+ 
+ 
+ *ans = y1*xb[x-1] + y2*xb[x] ;  
+
+  free(xa) ; 
+  free(xb) ; 
+  free(ind) ; 
+    
+  return x ;
+
+}
+double quartile(double *x, int n, double q) 
+{
+  double *a, *b, val, ans ; 
+  int i;
+
+  if (n==1) return b[0] ; 
+
+  ZALLOC(a, n, double) ;  
+  ZALLOC(b, n, double) ;  
+
+  for (i = 0; i < n; i++) {
+    a[i] = (double) i;
+  }
+
+  val = q * (double) (n-1) ; 
+  copyarr(x, b, n) ; 
+  sortit(b, NULL, n) ;
+
+  qinterp(a, b, n, val, &ans) ;
+
+  free(a) ; 
+  free(b) ; 
+
+  return ans ; 
+
+
+
+ free(a) ; 
+
+}
+
+double poissloglike (int kk, double mean)
+// log likelihood
+{
+  double y;
+
+  if (kk == 0) return -mean ;   // 0 log 0 = 0 here 
+  if (mean <= 0) fatalx("(poissloglike) zero mean!\n") ;
+
+  y = -mean + (double) kk *log (mean);
+  y -= logfac (kk);
+
+  return y;
+}
+
+
+double poissexp(int k, double mean)
+// expected value of # k hits mean m
+{
+ double y ; 
+ y = -mean ; 
+ y += (double) k * log (mean) ; 
+ y -= logfac(k) ; 
+
+ return exp(y) ; 
+
+}
+
+double wynn(double *v, int n, double *acc, int *nacc)
+{
+ double *x0, *x1, *xn, y ;
+ double t, amax, amin, tlast, normlim ;
+ int iter = 0, j, nn, isok  ;
+
+ normlim = vnorm(v,n) * 1.0e-8  ; 
+ vmaxmin(v, n, &amax, &amin) ;
+ if (amax<=amin) {
+  vclear(acc, amax, n/2) ;
+  *nacc = n/2 ;
+  return amax ;
+ }
+
+ ZALLOC(x0, n, double) ;
+ ZALLOC(x1, n, double) ;
+ ZALLOC(xn, n, double) ;
+ copyarr(v, x1, n) ;
+ nn = n ;
+ tlast = acc[0] = v[n-1] ; 
+ for (;;) {
+  for (j=0; (j+1) < nn ; ++j) {
+   isok = YES ;
+   y = x1[j+1] - x1[j] ; 
+   if (fabs(y) < normlim) { 
+    isok = NO ;  break ; 
+   }
+   t = x0[j+1] + 1.0/y ;
+   xn[j] = t ;
+  }
+  if (isok == NO) break ;
+  --nn ;
+  if (nn<2) break ;
+  copyarr(x1, x0, n) ;
+  copyarr(xn, x1, n) ;
+
+  for (j=0; (j+1) < nn ; ++j) {
+   isok = YES ;
+   y = x1[j+1] - x1[j] ; 
+   if (fabs(y) < normlim) { 
+    isok = NO ;  break ; 
+   }
+   t = x0[j+1] + 1.0/y ;
+   xn[j] = t ;
+  }
+  if (isok == NO) break ;
+  --nn ;
+  if (nn<2) break ;
+  copyarr(x1, x0, n) ;
+  copyarr(xn, x1, n) ;
+  tlast = acc[iter] = t ;
+  ++iter ;
+ }
+ free(x0) ; free(x1) ; free(xn) ;
+ *nacc = iter ;
+ return tlast ;
+}
+
+double *vwynn(double **vv, int n, int dim, double **acc, int *nacc)
+{
+ double **x0, **x1, **xn, *tt, *ans ;
+ double t, amax, amin, tlast, normlim ;
+ int iter = 0, j, nn, isok   ;
+ double *ww, *ww2, y ; 
+
+ ans = tt = NULL ;
+ *nacc = 0 ;   
+ if (n==0) return ans ; 
+ ans = tt = vv[n-1] ;
+ normlim = 0 ; 
+ for (j=0; j<n; ++j) { 
+  normlim += vnorm(vv[j], dim) ; 
+ }
+ normlim *= (1.0e-8) / (double) n ;  
+ x0 = initarray_2Ddouble(n, dim, 0) ;
+ x1 = initarray_2Ddouble(n, dim, 0) ;
+ xn = initarray_2Ddouble(n, dim, 0) ;
+ ZALLOC(ww, dim, double) ;
+ ZALLOC(ww2, dim, double) ;
+
+ copyarr2D(vv, x1, n, dim) ;
+ nn = n ;
+ for (;;) {
+  for (j=0; (j+1) < nn ; ++j) {
+   vvm(ww, x1[j+1], x1[j], dim) ; 
+   vabs(ww2, ww,  dim) ; 
+   vmaxmin(ww2, dim, NULL, &y) ; 
+   isok = YES ;
+   if (y < normlim) { 
+    isok = NO ;  break ; 
+   }
+   vin(ww, ww, dim) ;
+   vvp(xn[j], x0[j+1], ww, dim) ; 
+    tt = xn[j] ; 
+  }
+  if (isok == NO) break ;
+  --nn ;
+  if (nn<2) break ;
+  copyarr2D(x1, x0, n, dim) ;
+  copyarr2D(xn, x1, n, dim) ;
+
+  for (j=0; (j+1) < nn ; ++j) {
+   vvm(ww, x1[j+1], x1[j], dim) ; 
+   vabs(ww2, ww,  dim) ; 
+   vmaxmin(ww2, dim, NULL, &y) ; 
+   isok = YES ;
+   if (y < normlim) { 
+    isok = NO ;  break ; 
+   }
+   vin(ww, ww, dim) ;
+   vvp(xn[j], x0[j+1], ww, dim) ; 
+   tt = xn[j] ; 
+  }
+  if (isok == NO) break ;
+  --nn ;
+  if (nn<2) break ;
+  copyarr2D(x1, x0, n, dim) ;
+  copyarr2D(xn, x1, n, dim) ;
+  copyarr(tt, acc[iter], dim) ; 
+  ans = tt = acc[iter] ; 
+  ++iter ;
+ }
+ free2D(&x0, n) ; free2D(&x1, n) ; free2D(&xn, n);
+ free(ww) ;
+ free(ww2) ;
+ *nacc = iter ;
+
+ return ans ;
+}
+
+double rad2deg(double rad) 
+{
+  return rad*360.0/(2.0*pi()) ; 
+}
+
+double deg2rad(double deg) 
+{
+  return deg*2.0*pi()/360.0 ; 
+
+}
+
+
+double truncexpmean(double m, double thresh, int isupper) 
+// mean of truncated exponential of mean m 
+{
+
+
+  double t, x, z ; 
+
+  if (m==0) return 0 ; 
+
+  t = thresh/m ; 
+
+
+  if (isupper) return m*(t+1.0) ; 
+
+  z = exp(-t) ; 
+  x = 1 - t*z/(1-z) ; 
+
+  return m*x ; 
+
+
+}
+
+void mannwhit(double *a, int na, double *b, int nb, double *pu, double *pz, double *ppv) 
+// we shold probably do fast permutation test, esp. if na or nb small. 
+{
+  double *w, ya, yb, yu, yr , y, ym ; 
+  int *ord, *ind ; 
+  int t, k, x ; 
+
+  ZALLOC(w, na+nb, double) ; 
+  ZALLOC(ord, na+nb, int) ; 
+  ZALLOC(ind,  na+nb, int) ; 
+
+  ivclear(ord, 2, na+nb) ; 
+  ivclear(ord, 1, na) ; 
+
+  copyarr(a, w, na) ; 
+  copyarr(b, w+na, nb) ;
+
+
+  jitter(w, w, na+nb) ; 
+  sortit(w, ind, na+nb) ; 
+  t = 0 ; 
+  for (k=0; k<(na+nb); ++k) { 
+   x = ind[k] ; 
+   if (ord[x] == 2) continue ; 
+   t += k+1 ;
+   printf("zz %d %d\n", k, x) ; 
+  } 
+  yr = t ; 
+  y = (double) na*(na+1) ; 
+  *pu = yu = yr -y/2 ;       
+  y = (double) na*nb*(na+nb+1) ; 
+  y /= 12.0 ; y = sqrt(y) ; // s. dev 
+  ym = (double) (na*nb) ; ym /= 2 ; 
+  printf("zzums %9.3f  %9.3f %9.3f\n", yu, ym, y) ; 
+  ya = yu - ym ;  
+  *pz = ya/y ; 
+  *ppv = ntail(*pz) ; 
+  
+  free(w) ; 
+  free(ord) ; 
+  free(ind) ; 
+
+
+}
+void jitter(double *xout, double *xin, int n)  
+{
+ double y ; 
+ double *w ;
+
+ dither(xout, xin, n) ; 
+
+}
